/* 
 * PDFlib PLOP DS:
 * Encrypt file with certificate security.
 */

using System;
using System.Text;
using PLOP_dotnet;

struct recipients
{
    public string cert;
    public string perm;

    // Constructor:
    public recipients(string cert, string perm)
    {
        this.cert = cert;
        this.perm = perm;
    }
}

class Certsec
{
    static void Main(string[] args) 
    {
#if NET40
        const string data_path = "../data";
#else
        const string data_path = "../../data";
#endif
        // This is where the data files are. Adjust as necessary.
        const string searchpath_optlist = "searchpath={" + data_path + "}";

        /* Recipient certificates and permissions */
        recipients[] recipients = {
            /* no permission restrictions */
            new recipients(data_path + "/demo_recipient_1.pem", ""),

            /* form filling and signing permissions granted, but not others */
            new recipients(data_path + "/demo_recipient_2.pem",
                                "nomodify nocopy noannots noassemble"),
            new recipients(data_path + "/demo_recipient_3.pem",
                                "nomodify nocopy noannots noassemble"),
        };
        int RECIPIENTCOUNT = recipients.Length;


        if (args.Length < 2)
        {
#if NET40
            Console.WriteLine("usage: certsec <infile> <outfile>");
#else
            Console.WriteLine("usage: dotnet run <infile> <outfile>");
#endif
            return;
        }

        string input_file = args[0];
        string output_file = args[1];
        PLOP plop = null;

        try 
        {
            int doc, r;
            StringBuilder optlist = new StringBuilder();
            plop = new PLOP();

            plop.set_option(searchpath_optlist);

            /* open unprotected input file */
            if ((doc = plop.open_document(input_file, "")) == -1)
            {
                Console.WriteLine("Error: {0}", plop.get_errmsg());
                return;
            }


            /* add recipients */
            for (r=0; r<RECIPIENTCOUNT; r++)
            {
                optlist.Length = 0;
                optlist.AppendFormat("certificate={{filename={0}}} permissions={{{1}}}", recipients[r].cert, recipients[r].perm);
                if (plop.add_recipient(optlist.ToString()) == -1)
                {
                    /* emit warning and continue */
                    Console.WriteLine("Warning: {0}\n", plop.get_errmsg());
                }
            }


            /* create protected output file */
            if (plop.create_document(output_file, "input=" + doc) == -1)
            {
                Console.WriteLine("Error: {0}", plop.get_errmsg());
                return;
            }

            /* create protected output file */
            plop.close_document(doc, "");

        }
        catch (PLOPException e) 
        {
            Console.WriteLine("PLOP exception occurred in certsec sample:");
            Console.WriteLine("[{0}] {1}: {2}", e.get_errnum(),
                e.get_apiname(), e.get_errmsg());
        } finally {
            if (plop != null) {
                plop.Dispose();
            }
        }
    }
}
