// PDFlib PLOP DS:
// generate a dynamic visualization document in memory with PDFlib,
// and digitally sign a second document, using the dynamic document
// for signature visualization.

// #define USE_PKCS11

using System;
using PLOP_dotnet;
using System.Text;
using PDFlib_dotnet;

class dynamicsign
{
    static void Main(string[] args)
    {
        // Basic signature options
        const string sign_opts =
            "ltv=try " +        // use ltv=full to ensure LTV status
#if USE_PKCS11
            "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
            "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "
#endif
;
        System.DateTime ltime;
        StringBuilder timebuf = new StringBuilder();
        string textoptlist;
        string optlist;

        string[] months = { "January", "February", "March", "April", 
                            "May", "June", "July", "August", "September", 
                            "October", "November", "December" };

        // This is where the data files are. Adjust as necessary.
        const string searchpath_optlist = "searchpath={../data ../../data}";

        if (args.Length < 2)
        {
#if NET40
            Console.WriteLine("usage: dynamicsign <infile> <outfile>");
#else
            Console.WriteLine("usage: dotnet run <infile> <outfile>");
#endif
            return;
        }

        // ---------- Step 1: create visualization document in memory with PDFlib  ----------

        PDFlib p = new PDFlib();

        /* create a new PDF document in memory */
        try 
        {
            if (p.begin_document("", "") == -1)
            {
                Console.WriteLine("Error: {0}: {1}", p.get_apiname(), p.get_errmsg());
                return;
            }

            p.set_info("Creator", "dynamicsign.cs");
            p.set_info("Author", "Rainer Pl�ckl");
            p.set_info("Title", "Dynamic signature visualization document");

            p.begin_page_ext(300, 50, "");
            // Prepare dynamic text for the signature visualization 

            ltime = System.DateTime.Now;
            timebuf.Length = 0;
            timebuf.AppendFormat("Digitally signed {0} {1}, {2} at {3}:{4}:{5}", 
                months[ltime.Month - 1], ltime.Day, ltime.Year,
                ltime.Hour, ltime.Minute, ltime.Second);

            textoptlist = "fontsize=10 fontname=Helvetica encoding=unicode fillcolor=red";
            p.fit_textline(timebuf.ToString(), 10, 10, textoptlist);
            p.end_page_ext("");

            p.end_document("");
        }
        catch (PDFlibException e) 
        {
            Console.WriteLine("PDFlib exception occurred in hellosign sample:");
            Console.WriteLine("[{0}] {1}: {2}\n", e.get_errnum(), e.get_apiname(), e.get_errmsg());
        }

        // fetch the memory buffer containing the PDF document
        byte[] contents = p.get_buffer();
        PLOP plop = null;

        // ------------------ Step 2: sign the document -----------------
        try
        {
            int doc, visdoc;
            plop = new PLOP();

            plop.set_option(searchpath_optlist);

            // Create a virtual PVF file from the document data in memory
            plop.create_pvf("/pvf/visdoc.pdf", contents, "");

            if ((doc = plop.open_document(args[0], "")) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            // open visualization document from PVF file
            if ((visdoc = plop.open_document("/pvf/visdoc.pdf", "")) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }
            // Add field option for visualization to base signature options

            optlist = sign_opts + " field={visdoc=" + visdoc + " rect = {10 10 adapt adapt}}";
            if (plop.prepare_signature(optlist) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            // sign the output file
            if (plop.create_document(args[1], "input=" + doc) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            // close the input document
            plop.close_document(doc, "");

            // close the visualization document
            plop.close_document(visdoc, "");

            // delete PVF file containing input document
            if (plop.delete_pvf("/pvf/visdoc.pdf") == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }
        }
        catch (PLOPException e)
        {
            Console.WriteLine("PLOP exception occurred in dynamicsign sample:");
            Console.WriteLine("[{0}] {1}: {2}\n", e.get_errnum(),
                e.get_apiname(), e.get_errmsg());
        }
        finally
        {
            if (plop != null)
            {
                plop.Dispose();
            }

            if (p != null)
            {
                p.Dispose();
            }
        }
    }
}
