// PDFlib PLOP DS:
// generate a document in memory with PDFlib and sign it with PLOP DS
//
// For testing PKCS#11 set the following define:
// #define USE_PKCS11

using System;
using PLOP_dotnet;
using PDFlib_dotnet;

class HelloSign
{
    static void Main(string[] args) 
    {
        const string out_filename = "hellosign.pdf";

        // Basic signature options
        const string sign_opts =
            "ltv=try " +        // use ltv=full to ensure LTV status
#if USE_PKCS11
            "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
            "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "
#endif
            ;

        // This is where the data files are. Adjust as necessary.
        const string searchpath_optlist = "searchpath={../data ../../data}";

        // ---------- Step 1: generate document in memory with PDFlib ----------

        PDFlib p = new PDFlib();

        try 
        {

            /* create a new PDF document in memory */
            if (p.begin_document("", "") == -1)
            {
                Console.WriteLine("Error: {0}: {1}", p.get_apiname(), p.get_errmsg());
                return;
            }

            p.set_info("Creator", "hellosign.c");
            p.set_info("Author", "Rainer Plöckl");
            p.set_info("Title", "Hello, world/signed (.NET/C#)!");

            int font = p.load_font("Helvetica-Bold", "unicode", "");

            p.begin_page_ext(595, 842, "");

            p.setfont(font, 24);
            p.set_text_pos(50, 700);
            p.show("Hello, world/signed!");
            p.continue_text("(says .NET/C#)");
            p.end_page_ext("");

            p.end_document("");
        }
        catch (PDFlibException e) 
        {
            Console.WriteLine("PDFlib exception occurred in hellosign sample:");
            Console.WriteLine("[{0}] {1}: {2}\n", e.get_errnum(), e.get_apiname(), e.get_errmsg());
        }

        // fetch the memory buffer containing the PDF document
        byte[] contents = p.get_buffer();
        PLOP plop = null;

        try 
        {
            int doc;
            plop = new PLOP();

            plop.set_option(searchpath_optlist);

            // Create a virtual PVF file from the document data in memory
            plop.create_pvf("/pvf/hello.pdf", contents, "");

            // open unsigned input document from PVF file
            if ((doc = plop.open_document("/pvf/hello.pdf", "")) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            if (plop.prepare_signature(sign_opts) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            if (plop.create_document(out_filename, "input=" + doc) == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }

            // close the input document
            plop.close_document(doc, "");

            // delete PVF file containing input document
            if (plop.delete_pvf("/pvf/hello.pdf") == -1)
            {
                Console.WriteLine("Error: {0}: {1}", plop.get_apiname(), plop.get_errmsg());
                return;
            }
        }
        catch (PLOPException e) 
        {
            Console.WriteLine("PLOP exception occurred in hellosign sample:");
            Console.WriteLine("[{0}] {1}: {2}\n", e.get_errnum(), e.get_apiname(), e.get_errmsg());
        } finally {
            if (plop != null)
            {
                plop.Dispose();
            }

            if (p != null)
            {
                p.Dispose();
            }
        }
    }
}
