/* 
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 *  mini sample for inserting XMP metadata in PDF documents
 *
 */

using System;
using System.Text;
using PLOP_dotnet;

class insertxmp
{
    static void Main(string[] args)
    {
        /* Directory where files will be searched */
        const string searchpath = "../data ../../data";

        /* input document */
        const string in_filename = "PLOP-datasheet.pdf";

        /* output document */
        const string out_filename = "PLOP-datasheet-xmp.pdf";

        /* Option list for the generated output document.
         * Some alternatives for the option list using the supplied XMP samples:
         *
         * Insert custom metadata for the GWG (Ghent Workgroup)
         * Ad Ticket scheme (see http://www.gwg.org/Jobtickets.phtml):
         * "metadata={filename=gwg_ad_ticket.xmp validate=xmp2004}"
         *
         * Insert custom metadata based on Acme's enterprise schema:
         * "metadata={filename=acme.xmp}"
         *
         * Insert a custom "engineering" schema for scanned documents:
         * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
         * and requires XMP input which conforms to PDF/A-1:
         * "metadata={filename=engineering.xmp}"
         *
         * Insert a PDF/A extension schema after validating the XMP for PDF/A-1.
         * This works best with a PDF/A-1 input document (e.g. mini_pdfa1.pdf),
         * and requires XMP input which conforms to PDF/A-1:
         * "metadata={filename=machine_pdfa1.xmp validate=pdfa1}"
         *
         */

        const string out_optlist = "metadata={filename=gwg_ad_ticket.xmp}";
        PLOP plop = null;

        try {
            int doc;
            string optlist;
            plop = new PLOP();

            optlist = "searchpath={" + searchpath + "}";
            plop.set_option(optlist);

            /* open input file */
            doc = plop.open_document(in_filename, "");
            if (doc == -1)
            {
                Console.WriteLine("Error: {0}\n", plop.get_errmsg());
                return;
            }

            /* create the output file and add XMP metadata */
            if (plop.create_document(out_filename, out_optlist + " input=" + doc) == -1)
            {
                Console.WriteLine("Error: {0}\n", plop.get_errmsg());
                return;
            }

            /* close output file */
            plop.close_document(doc, "");

        }   catch (PLOPException e) {
            Console.WriteLine("PLOP exception occurred in insertxmp");
            Console.WriteLine("[{0}] {1}: {2}\n", e.get_errnum(),
                                        e.get_apiname(), e.get_errmsg());
        } finally {
            if (plop != null) {
                plop.Dispose();
            }
        }
    }
}
